//
// bidwatcher
// copyright (c) 1999 
// Trent McNair (trent@rmci.net)
// Tom McNair  (tmcnair@cyberhighway.net)
// 
// use of this code is restricted to the terms
// of the GPL, which should have been included in this
// distribution. If not, see www.gnu.org/copyleft/gpl.html.
// Here is the short version:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
#ifndef AUCTION_H
#define AUCTION_H
#include <qpopupmenu.h>
#include <qtooltip.h>
#include <qpainter.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qbitmap.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdialog.h>
#include <qdatetime.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qmultilinedit.h>

const int MAXBUFF = 5000;
const int MAXADS  = 100;
const int BIGBUFF = 7800;
const int HUGEBUFF = 50000;
const int NAMELEN = 66;
const int MAXAUCS = 100;
const int DELAY = 60000;
const int BIGDELAY = 7200000;  
const char versionInfo[]= "1.0.4b";
//const char theAddress[] = { 216, 32, 120, 178, 'c','g','i','-','n','e','w','.','e','b','a',
//                           'y','.','c','o','m','\0' };
const char theAddress[] = { 216, 32, 120, 136, 'c','g','i','.','e','b','a',
                           'y','.','c','o','m','\0' };
const char theAddress3[] = { 216, 32, 120, 139, 'c','g','i','3','.','e','b','a',
                           'y','.','c','o','m','\0' };

typedef char string30[30];

// Stucture to store info from web page
struct Advert_type {
   char Title1[80];      // text of Advertisement
   char Title2[80];      // more text
   char Url[228];    // the url of the Advertisement
   char Flag;          // option of printing the ad in bold / other color
};

//
// Stucture to store auction information
//
struct BidMakeup_type {
  char ItemNumber[30];     /*  Item Number (User Entered)    */
  char Description[129];   /*  Description Of Item For sale  */
  char CurrentBid[15];     /*  1: Current Bid                */
  char FirstBid[15];       /*  2: First (or starting) Bid    */
  char Quantity[15];       /*  3: Quantity of Items For Sale */
  char BidCount[15];       /*  4: Total Number of Bids       */
  char TimeLeft[51];       /*  5: Time Left in Auction       */
  char Location[101];      /*  6: Location of Item           */
  char Started[51];        /*  7: Time when Auction Started  */
  char Ends[51];           /*  8: Time/Data Auction Ends     */
  char Seller[76];         /*  9: User Selling the Item      */
  char HighBidder[76];     /* 10: Current High Bidder        */
  char BidInc[15];         /* 11: Minimum Bid Increment      */
  char Lowest[15];         /* 12: Minumum Bid Allowed now    */
  char SellerRate[15];
  char BidderRate[15];
  char snipeAmount[15];
  char snipeQty[15];
  char snipeKey[100];
  long int EndsValue;      // this is the time that the auction ends in seconds.
  char stat;
  char reserveMet;         /* y = met, n = not met, x = no reserve */ 
  bool isSnipe;
};
///////////////////////////////////////////////////////////////////////////////
//////////////////--------MyListBoxItem Class--------//////////////////////////
///////////////////////////////////////////////////////////////////////////////
class MyListBoxItem : public QListBoxItem
{
public:
   MyListBoxItem( const char *s, const QPixmap p,  QColor textC )
      : QListBoxItem(), pm(p), textColor(textC)
      { setText( s ); }

protected:
   virtual void paint( QPainter * );
   virtual int height( const QListBox * ) const;
   virtual int width( const QListBox * ) const;
   virtual const QPixmap *pixmap() { return &pm; }
   
private:
   QPixmap pm;
   QColor  textColor;
};
///////////////////////////////////////////////////////////////////////////////
///////////////-----------NewListBox Class-----////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class NewListBox : public QListBox
{
   Q_OBJECT
public:
   NewListBox( QWidget * parent=0, const char * name=0, WFlags f=0 ); 
signals:
   void RightClicked();
protected:
   void mousePressEvent( QMouseEvent * );
};
///////////////////////////////////////////////////////////////////////////////
/////////////---------BidWindow Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class BidWindow : public QDialog
{
   Q_OBJECT
public:
   BidWindow(QWidget *, const char *, bool , WFlags);
   void fillBlanks(BidMakeup_type &, int idx, char *, char *, int);
   struct BidMakeup_type theAuction;
signals:
   void setsnipe(BidMakeup_type &, int);
public slots:
   void CmCancel();
   void CmBidNow();
   void CmSnipe();
    void timerUp();
protected:
private:
   QLabel * itemLabel;
   QLabel * descriptionLabel;
   QLabel * sideBarLabel;
   QLabel * priceLabel;
   QLabel * categoryLabel;
   QLabel * infoLabel;
   QLabel * timeLabel;
   QTimer * DETAIL_TIMER;
   QLineEdit * maxBid;
   int theAuctionIdx;
   char userID[76];
   char userPASS[76];
   bool doSnipe;
   QPushButton * cancelBidButton;
   QPushButton * bidButton;
   QPushButton * snipeButton;
   QPalette        grayPalette;
	QPalette        blackPalette;
   QPalette        redPalette;
   int ebayOffset;
}; 
///////////////////////////////////////////////////////////////////////////////
//////////////---------GetNumber Class--------------------------//////////////
///////////////////////////////////////////////////////////////////////////////
class QGetNumber : public QDialog
{
   Q_OBJECT
public:
  	QGetNumber(QWidget * parent=0 , const char * name=0 , bool modal=FALSE, WFlags f=0 );
	~QGetNumber();
	
public slots:
   void CmOK();
   void CmCancel();

signals:
   void submit_number(char *);

private slots:

protected:
   void closeEvent( QCloseEvent * );
   
private:
   QLineEdit * inputEdit;
   QLabel * inputLabel;
   QPushButton * OKButton;
   QPushButton * CancButton;
};
///////////////////////////////////////////////////////////////////////////////
//////////////---------ConfigMenu Class--------------------------//////////////
///////////////////////////////////////////////////////////////////////////////
class ConfigMenu : public QDialog
{
   Q_OBJECT
public:
  	ConfigMenu(QWidget * parent=0 , const char * name=0 , bool modal=FALSE, WFlags f=0 );
	~ConfigMenu();
	void LoadValues(char *, char *, char *, bool , bool ,bool, bool, bool, int, char *);	
public slots:
   void CmKeep();
   void CmScrap();
signals:
   void its_done(char * , char * , char *, bool , bool , bool , bool, bool,  int, char * );
private slots:
protected:   
    void closeEvent( QCloseEvent * );
signals:
private:
   QLineEdit * userEdit;
   QLineEdit * passEdit;
   QLineEdit * browserEdit;
   QLineEdit * emailEdit;
   QSpinBox *  timeSpinner;
   QLabel * timeLabel;
   QLabel * browserLabel;
   QLabel * emailLabel;
   QLabel * userLabel;
   QLabel * passLabel;
   QCheckBox * listCheck;
   QCheckBox * bidCheck;
   QCheckBox * alarmCheck;
   QCheckBox * startCheck;
   QCheckBox * deleteCheck;
   QPushButton * scrapButton;
   QPushButton * keepButton;
};
///////////////////////////////////////////////////////////////////////////////
/////////////---------DetailsWindow Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class DetailsWindow : public QDialog
{
   Q_OBJECT
public:
   DetailsWindow(QWidget *, const char *, bool , WFlags);
   void fillBlanks(struct BidMakeup_type,char*,char*, int);
signals:
   void sendEmail(char *);
public slots:
   void CmClose();
   void CmEmailBidder();
   void CmEmailSeller();
   void timerUp();
protected:
private:
   //QLabel * auctionLabel;
   //QLabel * snipeLabel;
   QLabel * itemLabel;
   QLabel * descriptionLabel;
   QLabel * sideBarLabel;
   QLabel * priceLabel;
   QLabel * categoryLabel;
   QLabel * infoLabel;
   QLabel * timeLabel;
   QTimer * DETAIL_TIMER;
   struct BidMakeup_type theAuction;
   char   tempID[76];
   char   tempPASS[76];
   QPushButton * closeButton;
   QPushButton * emailBidderButton;
   QPushButton * emailSellerButton;
   QPalette        grayPalette;
	QPalette        blackPalette;
   QPalette        redPalette;
   int ebayOffset;
   void DoEmail(char*);
}; 
///////////////////////////////////////////////////////////////////////////////
/////////////---------QViewFile Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class  QFileView : public QDialog
{
   Q_OBJECT
public:
   QFileView(QWidget *, const char *, bool , WFlags);
   void fillBlanks(char * fileToView, bool isItThisMonth);
signals:
public slots:
   void CmClose();
   void CmChangeMonth();
protected:
private:
   QLabel * topLabel;
   QPushButton * closeButton;
   QPushButton * monthButton;
   QMultiLineEdit * viewArea;
   bool thisMonth; // TRUE = this month, FALSE = last month
}; 
///////////////////////////////////////////////////////////////////////////////
/////////////---------QEndedView Class----------////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
class  QEndedView : public QDialog
{
   Q_OBJECT
public:
   QEndedView(QWidget *, const char *, bool , WFlags);
   void fillBlanks(char * fileToView, bool isItThisMonth);
signals:
public slots:
   void CmClose();
   void CmChangeMonth();
protected:
private:
   QLabel * topLabel;
   QPushButton * closeButton;
   QPushButton * monthButton;
   QMultiLineEdit * viewArea;
   bool thisMonth; // TRUE = this month, FALSE = last month
}; 
//////////////////////////////////////////////////////////////////////////////
//////////------------BidWatcher Class--------------////////////////////////
//////////////////////////////////////////////////////////////////////////////
class BidWatcher : public QWidget
{
    Q_OBJECT
public:
    BidWatcher( QWidget *parent=0, const char *name=0 );
	~BidWatcher();
private slots:
   void CmUser();
   void CmAddItem();
   void CmTimeSync();
   void CmUpdate();
   void CmHelp();
   void CmExit();
   void CmCancelClicked();
   void changeName(char *, char *,char *, bool, bool, bool, bool, bool, int, char *);
   void doubleClick(int);
   void addNewItem(char *);
   void timer1Up();
   void timer2Up();
   void timer3Up();
   void timer4Up();
   void CmMenu();
   void CmDetails();
   void CmViewFile();
   void CmViewEnded();
   void UpdateItem();
   void deleteItem();
   void flush();
   void setupSnipe(BidMakeup_type &,int);
   void CmBid();
   void CmUnBid();
   void ClearSnipe(int);
   void CmWeb();
   void CmEmail(char*);
protected:
   void resizeEvent( QResizeEvent * );
signals:
private:
	NewListBox      * aucList;
	QLabel          * brightLabel;
	QLabel          * statLabel;
	QLabel          * timeLabel;
   QLabel          * columnHeaderLabel;
	QToolButton     * cancelButton;
	QToolButton     * UserButton;
	QToolButton     * AddButton;
	QToolButton     * ClockButton;
	QToolButton     * UpdateButton;
	QToolButton     * ExitButton;
	QToolButton     * HelpButton;
	QTimer          * TIMER1;
	QTimer          * TIMER2;
	QTimer          * TIMER3;
	QTimer          * TIMER4;
	ConfigMenu      * firstMenu;
	QGetNumber      * getNumWin;
	QPopupMenu      * listSelect; 
	BidWindow       * bidWindow;
   DetailsWindow   * detailsWindow;
   QFileView       * viewFileWindow;
   QEndedView      * viewEndedWindow;
	
   int             aucIdx;
	bool            updateInProgress;
	bool            firstRun;
	bool            checkForSnipe;
	int             Timer1Idx;
	int             Timer2Idx;
	int             bidIdx;
	int             timeDiff;
   int             snipeDelay; 
   char            authID[76];
   char            authPASS[76];
	char            browserPATH[200];
   char            emailPATH[200];
   bool            trackListings;
   bool            trackBids;
   bool            autoDelete;
   bool            doStartup;
   bool            soundOn;
	bool            alarmOn;
	bool            clockIsSet;
	struct 		    BidMakeup_type BidMakeup[MAXAUCS+1];
	string30        BidList[MAXAUCS + 1];
	QPixmap         brightLed;
	QPixmap         dimLed;
	QPixmap         id_btn;
	QPixmap         add_btn;
	QPixmap         clock_btn;
	QPixmap         update_btn;
	QPixmap         help_btn;
	QPixmap         exit_btn;
	QPixmap         canc_btn;
	QPixmap			redDot;
	QPixmap         blueDot;
	QPixmap         blackDot;
	QBitmap         maskDot;
	QPixmap         greenDot;
	QPixmap         snipeDot;
	QBitmap         maskSnipe;
	QPalette        yellowPalette;
	QPalette        greenPalette;
	
	void            TimeSync();
	void            DoSnipe(int);
	void            UpdateListItem(int);
	void	        UpdateList();
   void          ArrangeList();
	void	        DeleteAuction(int);
	void 	        ReadaucFile();
	void            UpdateAll(bool);
	void            UpdateLatest();
	int 	        GetUserBids();
	int             GetUserListings();
	int             UpdateAuction(BidMakeup_type &);  
   void          WriteLog(int);      
   void           WriteAucFile();  	
};

#endif // AUCTION_H
