//
//  bidwatcher
//  copyright (c) 1998-2000
//  Trent McNair (trent@rmci.net)
//  Tom McNair  (tmcnair@cyberhighway.net)
// 
//  helpers.cpp:
//
//  Helper functions for bidwatcher, no platform/gui specific
//  code.
//
//  Patch to fix the jan 2000 ebay changes courtesy 
//  Jeffrey Veiss
//
char *CheckPrice="0123456789.$,";      // Characters allowed in a price
  

char *Months[]={
        "Jan",
        "Feb",
        "Mar",
        "Apr",
        "May",
        "Jun",
        "Jul",
        "Aug",
        "Sep",
        "Oct",
        "Nov",
        "Dec",
        "*"
};

char *SearchOps[]={ "Starts at",      // 1:  +1
               "Currently",           // 2:  +1                        
 
               "First bid",           // 3:  +1                        
 
               "Quantity",            // 4:  +1                        
 
               "# of bids",           // 5:  +1 -> Read until " " or "(" 
               "Time left",           // 6:  +1                        
 
               "Location",            // 7:  +1                        
 
               "Started",             // 8:  +1                        
 
               "Ends",                // 9:  +1                        
 
               "Seller (Rating)",              // 10: +1               
          
               "High bid",            // 11: +1                        
 
               "Bid increment",       // 12: +1                        
 
               "Lowest",              // 13: +1 
               "High bids",           // 14: +1                    
               "*"};
void calculateBidIncrement(const char * currentBid, char * newBid)
{
        // step 1 - parse currentBid into useable int's
   int err;
        char curBid[15];
        if ( currentBid[0] == '$' )
        {
                int limit = strlen(currentBid);
                for ( int i=0; i < limit; i++ )
                {
                        // strip the '$' off the currentBid;
                        curBid[i] = currentBid[i + 1];
                }
                curBid[14] = '\0'; // because I'm insane
        }
        else
        {
                strcpy(newBid, "no bids");
                return;
        }
        float fcb = atof(curBid);   // fcb = 'float current bid'
        float inc;
        if ( fcb < 1.00 ) inc = 0.05;
        else if ( fcb < 5.00 ) inc = 0.25;
        else if ( fcb < 25.00 ) inc = 0.50;
        else if ( fcb < 100.00 ) inc = 1.00;
        else if ( fcb < 250.00 ) inc = 2.50;
        else if ( fcb < 500.00 ) inc = 5.00;
        else if ( fcb < 1000.00 ) inc = 10.00;
        else if ( fcb < 2500.00 ) inc = 25.00;
        else if ( fcb < 5000.00 ) inc = 50.00;
        else inc = 100.00;
        fcb = fcb + inc;
        err = sprintf(newBid, "%f", fcb);
        // now, truncate the end:
        unsigned int j = 0;
        while ( newBid[j] != '.' )
        {
                j++;
                if ( j > strlen(newBid) ) break;
        }
        j+=3;
        newBid[j] = '\0';
        return;
}
///////////////////////////////////////////////////////////////////////////////
// StripHtmlTags()
//    -strips the tags off an html string
///////////////////////////////////////////////////////////////////////////////
void StripHtmlTags(const char * stringToStrip, char * Buff, int BuffSize)
{
   strcpy(Buff, "");
   int buffLength = strlen(stringToStrip);
   int BuffIdx = 0;
   int u,c;
   int IncludeFlag = 5;
   // strip the html tags
   for ( u = 0; (u < buffLength) && ( BuffIdx < BuffSize );u++)
   {
      c = stringToStrip[u];
      if (c=='<') IncludeFlag=5;
      else if (c=='>') IncludeFlag=10;
      else if ((IncludeFlag==10) && (BuffIdx<(BuffSize-1)))
      {
         if (((c>31) && (c<127)) || (c=='\n')) Buff[BuffIdx++]=c;
      }
   }
   Buff[BuffIdx - 1] = '\0';
   Buff[BuffSize - 1] = '\0';
   return;
}
//////////////////////////////////////////////////////////////////////
//  StripAndTab - strips html tags off an html buffer and inserts
//     tabs where each tag goes, this aids in parsing later
//////////////////////////////////////////////////////////////////////
void StripAndTab(const char * stringToStrip, char * Buff, int BuffSize)
{
   strcpy(Buff, "");
   int buffLength = strlen(stringToStrip);
   int BuffIdx = 0;
   int u,c;
   int IncludeFlag = 5;
   bool tabFlag = FALSE;
   for ( u = 0; (u < buffLength) && ( BuffIdx < BuffSize );u++)
   {
      c = stringToStrip[u];
      if (c=='<') {
         tabFlag = FALSE;
         IncludeFlag=5;
      }
      else if (c=='>') {
         tabFlag = TRUE;
         IncludeFlag=10;
      }
      else if ((IncludeFlag==10) && (BuffIdx<(HUGEBUFF-1)))
      {
         if ( tabFlag ) {
            Buff[BuffIdx++] = '\t';
            tabFlag = FALSE;
         }
         if (((c>31) && (c<127)) || (c=='\n')) Buff[BuffIdx++]=c;
      }
   }
  Buff[BuffIdx - 1] = '\0';
  Buff[BuffSize - 1] = '\0';
}                                                                      
            
//////////////////////////////////////////////////////////////////////////////////
// ProcessBidSubmission
//    takes the html response from submitting a bid and returns an appropriate
//    error code.
// return values:
//    1 - bid submission was a success - we are the high bidder
//    2 - Bid too low
//    3 - Quantity Problem
//    4 - Other problem (unknown, probably bad connection)
//   
//////////////////////////////////////////////////////////////////////////////////
int ProcessBidSubmission( char * Buff, char * lineBuff )
{
        istrstream streamBuff(Buff);
        bool foundKey = FALSE;
        bool foundEndOfForm = FALSE;
        bool foundAffirmation = FALSE;
        while ( streamBuff ) {
                streamBuff.getline(lineBuff, 1024, '\n');
                if ( strstr(lineBuff, "/FORM") != NULL ) {
                        foundEndOfForm = TRUE;
                        break;
                }
        }
        if ( !foundEndOfForm ) return 4;
        while ( streamBuff ) {
                streamBuff.getline(lineBuff, 1024, '\n');
                if ( strstr(lineBuff, "name=key") != NULL ) {
                        foundKey = TRUE;
                        if ( foundAffirmation ) return 1;
                }
                else if ( strstr(lineBuff, "Review bid for") != NULL ) {
                        foundAffirmation = TRUE;
                        if (foundKey) return 1;
                }
                else if ( strstr(lineBuff, "Problem with bid amount") != NULL ) return 2;
                else if ( strstr(lineBuff, "Problem with quantity") != NULL ) return 3;
        }
        return 4;
}
//////////////////////////////////////////////////////////////////////////////////
// ProcessBid()
//    takes the html response from making a bid
// return values:
//    1 - bid submission was a success - we are the high bidder
//    2 - bid successful, but we were outbid.
//    3 - bid under the current asking price
//    4 - problem with the bid quantity
//    5 - Auction is over.
//    6 - invalid username / password.
//    7 - unknown error
//   
//////////////////////////////////////////////////////////////////////////////////
int ProcessBid( char * Buff )
{
        char lineBuff[1024];
   istrstream streamBuff(Buff);
        bool foundEndOfForm = FALSE;
        while ( streamBuff ) {
                streamBuff.getline(lineBuff, 1024, '\n');
                if ( strstr(lineBuff, "/FORM") != NULL ) {
                        foundEndOfForm = TRUE;
                        break;
                }
        }
        if ( !foundEndOfForm ) return 7;
        while ( streamBuff ) {
                streamBuff.getline(lineBuff, 1024, '\n');
                if ( strstr(lineBuff, "are the current high bidder") != NULL ) return 1;
                else if ( strstr(lineBuff, "outbid you") != NULL ) return 2;
                else if ( strstr(lineBuff, "Problem with bid amount") != NULL ) return 3;
                else if ( strstr(lineBuff, "Problem with quantity") != NULL ) return 4;
                else if ( strstr(lineBuff, "Cannot proceed") != NULL ) return 5;
                else if ( strstr(lineBuff, "or password invalid") != NULL ) return 6;
        }
        return 7;
}
//////////////////////////////////////////////////////////////////////////////////
// ParseEmailAddress
//    parses the email address out of the web paged passed in Html.
// return values:
//    1 - success
//    2 - authorization failure
//    2 - unknown error
//////////////////////////////////////////////////////////////////////////////////
int ParseEmailAddress(char * Buff, char * emailAddress)
{
   char tempBuff[1024];
   if ( strstr(Buff, "password invalid") != NULL ) return 2;
   if ( strstr(Buff, "via e-mail:") == NULL ) return 3;
   // now the likely errors are weeded - so we can parse the 
   // email address out of Buff
   istrstream buffStream(Buff);
   while (buffStream)
   {
      buffStream.getline(tempBuff, 1024, '\n');
      if ( (strstr(tempBuff, "via e-mail:")) != NULL )
      {
         // parse the last little bit
         char * scratch;
         scratch = strtok(tempBuff,":");
         scratch = strtok(0,"\n");
         if ( scratch )
         {
            // now scrape off the leading space, there has to be a better way
            int limit = strlen(scratch);
            for ( int i =0; i < limit; i++ ) scratch[i] = scratch[i+1];
            strcpy(emailAddress, scratch);
         }
      }
   }
   return 1;
}
///////////////////////////////////////////////////////////////////////////////
// ParseList
//   parses auction information from the listings or bidders list pages.
///////////////////////////////////////////////////////////////////////////////
int ParseList( BidMakeup_type * TempList, char * Buff)
{
   char lineBuff[2048];
   int cnt;
   int cnt2;
   istrstream streamBuff(Buff);
   int listCount = 0;
   while( streamBuff )
   {
      streamBuff.getline(lineBuff, 2048, '\n');
      if ( (lineBuff[0]=='\t') && (isdigit(lineBuff[1])) && (isdigit(lineBuff[2])) && 
           (isdigit(lineBuff[3])) && (isdigit(lineBuff[4])) ) {
         cnt2 = 1;
         cnt=0;
         if ( strstr(lineBuff,"Hidden") == NULL )
         {
            char * scratch;
            scratch = strtok(lineBuff, "\t");
            if ( !scratch ) return listCount;
            strcpy(TempList[listCount].ItemNumber, scratch);

            scratch = strtok(0,"\t");
            if ( !scratch ) return listCount;
            scratch = strtok(0,"\t");
            if ( !scratch ) return listCount;
            strcpy(TempList[listCount].Ends, scratch);

            scratch = strtok(0, "\t");
            if ( !scratch ) return listCount;
            strcpy(TempList[listCount].CurrentBid, scratch);

            scratch = strtok(0,"\t");
            if ( !scratch ) return listCount;
            strcpy(TempList[listCount].Description, scratch);

            scratch = strtok(0, "\t");
            if ( !scratch ) return listCount;
            strcpy(TempList[listCount].HighBidder, scratch);

            // now, we have all the info up to high bidder.  Need to determine if there
            // is high bidder rate information.

            if ( strstr(TempList[listCount].HighBidder, "No Bids") != NULL ) {
               strcpy(TempList[listCount].CurrentBid, "No Bids");
            }
            else if ( strstr(TempList[listCount].HighBidder, "Dutch bids") != NULL ) {
               strcpy(TempList[listCount].CurrentBid, "Dutch" );
            }

            listCount++;
            if (listCount >= (MAXAUCS - 1)) break;
         }
      }
   }
   return listCount;
}
//////////////////////////////////////////////////////////////////////////////
// CheckBadChars
// aids GetAucInfo in parsing html correctly
//////////////////////////////////////////////////////////////////////////////
void CheckBadChars(char *Str2Chk, char *GoodChars) {

  int idx,idx2;

  for (idx=0;Str2Chk[idx]!=0;++idx) {

    for (idx2=0;GoodChars[idx2]!=0;++idx2) {
      if (GoodChars[idx2]==Str2Chk[idx]) break; 
    }

    if (GoodChars[idx2]==0) { 
      Str2Chk[idx]=0; 
      return;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
// ReturnMonthVarNum
//////////////////////////////////////////////////////////////////////////////
int ReturnMonthVarNum(char *mon)
{
  int idx=0;

  for(idx=0;Months[idx][0]!='*';++idx) 
    if (strcmp(mon,Months[idx])==0) return (idx);
  
  return -1;

}
//////////////////////////////////////////////////////////////////////////////
// ReturnBidVarNum
//////////////////////////////////////////////////////////////////////////////
int ReturnBidVarNum(char *LineData)
{
  int idx=0;

  for(idx=0;SearchOps[idx][0]!='*';++idx) 
    if (strcmp(LineData,SearchOps[idx])==0) return (idx+1);
  
  return 0;

}
//////////////////////////////////////////////////////////////////////////////
// ClearBidMakeUp:
// Clears the information in a BidMakeup struct
//////////////////////////////////////////////////////////////////////////////
void ClearBidMakeUp(struct BidMakeup_type & NewAuction) 
{
  NewAuction.ItemNumber[0]=0;
  NewAuction.Description[0]=0;
  NewAuction.CurrentBid[0]=0;
  NewAuction.FirstBid[0]=0;
  NewAuction.Quantity[0]=0;
  NewAuction.BidCount[0]=0;
  NewAuction.TimeLeft[0]=0;
  NewAuction.Location[0]=0;
  NewAuction.Started[0]=0;
  NewAuction.Ends[0]=0;
  NewAuction.Seller[0]=0;
  NewAuction.HighBidder[0]=0;
  NewAuction.BidInc[0]=0;
  NewAuction.Lowest[0]=0;
  NewAuction.SellerRate[0] = 0;
  NewAuction.BidderRate[0] = 0;
  NewAuction.snipeAmount[0] = 0;
  NewAuction.snipeQty[0] = 0;
  NewAuction.snipeKey[0] = 0;
  NewAuction.stat = '0';
  NewAuction.reserveMet = '0';
  NewAuction.isSnipe = FALSE;
  NewAuction.EndsValue = 0;
}
//////////////////////////////////////////////////////////////////////////////
// CalcEndsValue
//
// takes ebay format ending time (i.e. xx/xx/xx, xx:xx:xx) and converts
// it to time in seconds since a epoch (jan 1, 1970), this is for easy
// sorting later.
//
//////////////////////////////////////////////////////////////////////////////
long int CalcEndsValue( char * Ends )
{
   if ( strlen(Ends) > 12 )
   {
      //  convert Ends into in values of year, month,  day
      char tokEnds[50]; // the tokenized Ends, will be tossed.
      strncpy(tokEnds, Ends, 50);
      tokEnds[49] = '\0'; 
      char * scratch;
      int seconds,minutes,hours,month,day,year;

      scratch = strtok(tokEnds, "-");
      if ( scratch ) {
                        if ((month = ReturnMonthVarNum(scratch)) < 0) {
                                fprintf (stderr, "Date error");
                                exit (1);
                        }
                }
      else return -1;

      scratch = strtok(0,"-");
      if ( scratch )  day = atoi(scratch);
      else return -1;

      scratch = strtok(0," ");
      if ( scratch )  year = atoi(scratch);
      else return -1;

      scratch = strtok(0,":");
      if ( scratch ) hours = atoi(scratch);
      else return -1;

      scratch = strtok(0,":");
      if ( scratch )  minutes = atoi(scratch);
      else return -1;

      scratch = strtok(0," \0");
      if ( scratch )  seconds = atoi(scratch);
      else return -1;

      if ( year < 25 ) year = year + 100;

      struct tm TheTime;
      TheTime.tm_sec = seconds;
      TheTime.tm_min = minutes;
      TheTime.tm_hour = hours;
      TheTime.tm_mday = day;
      TheTime.tm_mon = month;
      TheTime.tm_year = year;
      TheTime.tm_yday = 0;
      TheTime.tm_wday = 0;
      TheTime.tm_isdst = -1;
         
      // now we have day,month,year,minutes,hours,seconds in integer format,
      // we just need to convert it to seconds since the epoch and we're in
      // business.
      long int the_time = mktime(&TheTime);
      return the_time;
   }
   else return -1; 
}
//////////////////////////////////////////////////////////////////////////////
//   CalcTimeLeft
//       calculates the time left in an auction in seconds.
//   stringTimeLeft
//       calculates the time left and return a string of that value
//////////////////////////////////////////////////////////////////////////////
long int CalcTimeLeft ( long int endTime, int ebayOffset )
{
   long int localTime = time(NULL);
   localTime += ebayOffset;
   return (endTime - localTime);
}

int stringTimeLeft ( long int endTime, int ebayOffset, char * chTimeLeft )
{
   long int localTime = time(NULL);
   localTime += ebayOffset;
   int intTimeLeft = (endTime - localTime);
   int err;
   if ( intTimeLeft >= 172800 ) {
      err = sprintf(chTimeLeft, "%d days, %d hrs", intTimeLeft / 86400,
                        (intTimeLeft % 86400) / 3600 );
                return intTimeLeft;
   }
   else if ( intTimeLeft >= 86400 ) {
      err = sprintf(chTimeLeft, "%d day, %d hrs", intTimeLeft / 86400,
                        (intTimeLeft % 86400) / 3600 );
                return intTimeLeft;
   }
   else if ( intTimeLeft >= 3600 ) {
      err = sprintf(chTimeLeft, "%d hrs, %d mins", intTimeLeft / 3600,
                        (intTimeLeft % 3600) / 60);
                return intTimeLeft;
   }
   else if ( intTimeLeft >= 60 ) {
      err = sprintf(chTimeLeft, "%d mins, %d secs", intTimeLeft / 60,
                        intTimeLeft % 60);
                return intTimeLeft;
   }
   else if ( intTimeLeft >= 0 ) {
      err = sprintf(chTimeLeft, "%d seconds", intTimeLeft );
      return intTimeLeft;
   }
   else {
      err = sprintf(chTimeLeft, "Auction Ended" );
      return intTimeLeft;
   }
}     
//////////////////////////////////////////////////////////////////////////////
//   GetAucInfo
//       loads up BidMakeup[] with auction info.
//////////////////////////////////////////////////////////////////////////////
bool GetAucInfo(char *Buff, struct BidMakeup_type & NewAuction)
{
   int idx=0;
   int cnt;
   char LineData[1025];
   char * scratch;
   char newName[76];

   // First Lets clear out the right bits of the structure
   NewAuction.CurrentBid[0] = 0;
   NewAuction.FirstBid[0] = 0;
   NewAuction.Quantity[0] = 0;
   NewAuction.BidCount[0] = 0;
   NewAuction.TimeLeft[0] = 0;
   NewAuction.Location[0] = 0;
   NewAuction.Started[0] = 0;
   NewAuction.Ends[0] = 0;
   NewAuction.Seller[0] = 0;
   NewAuction.HighBidder[0] = 0;
   NewAuction.Lowest[0] = 0;
   NewAuction.reserveMet = 'x';
   
   istrstream streamBuff(Buff);
   streamBuff.getline(LineData, 1024, '\n');
   if ( strstr(LineData,"eBay item") != NULL )
   scratch = strtok(LineData, ")");
   scratch = strtok(0, "\n");
   if (scratch)
   {
      while ( scratch[idx + 3] != '\0' )
     {
        NewAuction.Description[idx] = scratch[idx + 3];
       // strcpy(NewAuction.Description, scratch);
       idx++;
     }
     NewAuction.Description[idx] = '\0';
     idx=0;
   }
   while (streamBuff) 
   {
      streamBuff.getline(LineData, 1024, '\n');
    
      // The Idea here is to get the info we can get, if for
      // some reason some of the data is missing or out of
      // order we can still salvage what we can..
    
      if (LineData[0]!=0) 
      {
         switch (ReturnBidVarNum(LineData)) 
         {
        case 1 : if (NewAuction.CurrentBid[0]==0) {
                    streamBuff.getline(LineData, 1024, '\n');
                    if ( strstr(LineData,"reserve not") != NULL ) NewAuction.reserveMet = 'n';
                    else if ( strstr(LineData,"reserve met") != NULL ) NewAuction.reserveMet = 'y';
                  }
                  break;

        case 2 : if (NewAuction.CurrentBid[0]==0) {
                  streamBuff.getline(LineData, 1024, '\n');
                  if ( strstr(LineData,"reserve not") != NULL ) NewAuction.reserveMet = 'n';
                  else if ( strstr(LineData,"reserve met") != NULL ) NewAuction.reserveMet = 'y';
                   CheckBadChars(LineData,CheckPrice);
                   strncpy(NewAuction.CurrentBid,LineData,14);
                 }
                 break;

        case 3 : if (NewAuction.FirstBid[0]==0) {
                  streamBuff.getline(LineData, 1024, '\n');
                   CheckBadChars(LineData,CheckPrice);
                   strncpy(NewAuction.FirstBid,LineData,14);
                 }
                 break;

        case 4 : if (NewAuction.Quantity[0]==0) {
                   streamBuff.getline(LineData, 1024, '\n');
                   CheckBadChars(LineData,"0123456789");
                   strncpy(NewAuction.Quantity,LineData,14);
                 }
                 break;

        case 5 : if (NewAuction.BidCount[0]==0) {
                   streamBuff.getline(LineData, 1024, '\n');
                   for(cnt=0;(((LineData[cnt]!=' ') && (LineData[cnt]!=0)) && (cnt<14));++cnt)
                     NewAuction.BidCount[cnt]=LineData[cnt];
                   NewAuction.BidCount[cnt]=0;
                 }
                 break;

        case 6 : if (NewAuction.TimeLeft[0]==0) {
                   streamBuff.getline(LineData, 1024, '\n');
                   strncpy(NewAuction.TimeLeft,LineData,50);
                 }
                 break;

        case 7 : if (NewAuction.Location[0]==0) {
                  streamBuff.getline(LineData, 1024, '\n');
                   strncpy(NewAuction.Location,LineData,100);
                 }
                 break;

        case 8 : if (NewAuction.Started[0]==0) {
                   streamBuff.getline(LineData, 1024, '\n');
                   strncpy(NewAuction.Started,LineData,50);
                 }
                 break;

        case 9 : if (NewAuction.Ends[0]==0) {
                   streamBuff.getline(LineData, 1024, '\n');
                   strncpy(NewAuction.Ends,LineData,50);
                 }
                 break;

        case 10 : if (NewAuction.Seller[0]==0) {
                  streamBuff.getline(LineData, 1024, '\n');
                   strncpy(NewAuction.Seller,LineData,75);
                 }
                 break;

        case 11: if (NewAuction.HighBidder[0]==0) {
                   streamBuff.getline(LineData, 1024, '\n');
                   strncpy(NewAuction.HighBidder,LineData,76);
                   NewAuction.HighBidder[75] = '\0';
                 }
                 break;

        //case 12: idx=GetLineOfData(idx,1024,LineData,Buff);
        //         CheckBadChars(LineData,CheckPrice);
        //         strncpy(NewAuction.BidInc,LineData,14);
        //         break;

        case 13: streamBuff.getline(LineData, 1024, '\n');
                 CheckBadChars(LineData,CheckPrice);
                 strncpy(NewAuction.Lowest,LineData,14);
                 break;
             
      case 14: streamBuff.getline(LineData, 1024, '\n');
                 CheckBadChars(LineData,CheckPrice);
                 strcpy(NewAuction.CurrentBid,"Dutch");
             strcpy(NewAuction.HighBidder, "Dutch Auction");
                 break;

        default:;
      }
    }
  }

  if ( NewAuction.CurrentBid[0] == 0 ) strcpy(NewAuction.CurrentBid, "no bids");

  // now calculate the ending time in seconds and save it
  
  
  // now We'll parse the High Bidder and Seller's ratings off their names
  // and assign them to .SellerRate and .BidderRate

   strcpy ( newName, NewAuction.Seller);
   scratch = strtok(newName, " ");
   if ( !scratch ) return FALSE;
   strcpy( NewAuction.Seller, scratch);
   scratch = strtok(0,")");
   if ( scratch )
   {
      strcpy(NewAuction.SellerRate, scratch);
      strcat(NewAuction.SellerRate, ")");
   }

   if ( (strcmp(NewAuction.HighBidder, "Dutch Auction") == 0) ||
        (strcmp(NewAuction.HighBidder, "--") == 0) )
   {
       return TRUE;
   }
   else
   {
      strcpy( newName, NewAuction.HighBidder);
      scratch = strtok(newName, " ");
      if ( !scratch ) return FALSE;
      strcpy(NewAuction.HighBidder, scratch);
      scratch = strtok(0,")");
      if ( scratch )
      {
          strcpy(NewAuction.BidderRate, scratch);
          strcat(NewAuction.BidderRate, ")");
      }
   }
   return TRUE;
} // end GetAucInfo()
void MakeFileName(char * prefix1, char * prefix2, char * fileName, bool thisMonth)
{
   int month,year, err;
   time_t time_now = time(0);
   struct tm * timeStruct = localtime(&time_now);
   if ( thisMonth ) month = timeStruct->tm_mon + 1;
      else month = timeStruct->tm_mon;
   year = timeStruct->tm_year;
   if ( year > 99 ) year = year - 100;
   
   if ( year > 75 ) err = sprintf(fileName, "%s%s%d_%d.txt", prefix1, prefix2, month, year); 
   else err = sprintf(fileName, "%s%s%d_0%d.txt", prefix1, prefix2, month, year); 
}
void MakeClockTime(int offSet, char * theTime)
{
   time_t intTime = time(NULL) + offSet;
   //struct tm tmTime = *localtime(&intTime);
   //int err = sprintf(theTime,"%d:%d:%d", tmTime.tm_hour, tmTime.tm_min, tmTime.tm_sec);
   //return;
   char * tempTime = ctime(&intTime);
   char * scratch = strtok(tempTime, " ");
   if ( !scratch ) return;
   scratch = strtok(0, " ");
   if ( !scratch ) return;
   scratch = strtok(0, " ");
   if ( !scratch ) return;
   scratch = strtok(0, " ");
   if ( !scratch ) return;
   strcpy(theTime, scratch);
   return;
}
//////////////////////////////////////////////////////////////////////////////
// calcTimeDiff
//  
//  buffer is an unfiltered webpage from ebay containing the 
//  current time (http://cgi.ebay.com/aw-cgi/eBayISAPI.dll?TimeShow)
//
//  calcTimeDiff parses the time from the above mentioned page and 
//  calculates the difference between ebay time and local time, returning
//  the value in seconds.
//////////////////////////////////////////////////////////////////////////////
int calcTimeDiff ( char * buffer)
{  
   struct timeval time_tt;
   //time_t new_time_t = time(NULL);
   gettimeofday(&time_tt, 0);
   time_t new_time_t = time_tt.tv_sec;
   char chHr[9];
   char chMin[9];
   char chSec[9];
   char chYear[9];
   char chMonth[9];
   char chDay[9];
   char lineBuff[1024];
   char * scratch;
   istrstream streamBuff(buffer);
   struct tm ebay_time;
   ebay_time.tm_isdst = 0;
   while( streamBuff )
   {
      streamBuff.getline(lineBuff, 1024, '\n');
      if ( strstr(lineBuff, "Pacific Time") != NULL )
      {
         scratch = strtok(lineBuff, ">");
         if (scratch) scratch = strtok(0, ">");
         if (scratch) scratch = strtok(0, ">");
         if (scratch) scratch = strtok(0, " ");
         if (scratch) scratch = strtok(0, " ");
         if (scratch) strcpy(chMonth, scratch);

         scratch = strtok(0, " ");
         if (scratch) strcpy(chDay, scratch);

         scratch = strtok(0, " ");
         if (scratch) strcpy(chYear, scratch);

         scratch = strtok(0, ":");
         if (scratch) strcpy(chHr, scratch);

         scratch = strtok(0,":");
         if (scratch) strcpy(chMin, scratch);

         scratch = strtok(0," ");
         if (scratch) strcpy(chSec, scratch);
         break;
      }
   }
   ebay_time.tm_hour = (((int(chHr[strlen(chHr) - 2])) - 48) * 10) +
                       ((int(chHr[strlen(chHr) - 1])) - 48);
   ebay_time.tm_min =  (((int(chMin[strlen(chMin) - 2])) - 48) * 10) +
                       ((int(chMin[strlen(chMin) - 1])) - 48);
   ebay_time.tm_sec =  (((int(chSec[strlen(chSec) - 2])) - 48) * 10) +
                       ((int(chSec[strlen(chSec) - 1])) - 48);
   ebay_time.tm_mday = atoi(chDay); 
        if ((ebay_time.tm_mon = ReturnMonthVarNum(chMonth)) < 0) {
                fprintf (stderr, "Date error");
                exit (1);
        }
   ebay_time.tm_year = atoi(chYear) - 1900;  

   time_t theTime = mktime(&ebay_time) - new_time_t;

   return theTime;
}
//////////////////////////////////////////////////////////////////////////////
// parseRows
//   parses rows from raw html and puts them each on a line (if it
//   finds a newline character in a line it converts it to a space).
//////////////////////////////////////////////////////////////////////////////
void parseRows(const char * fromBuff, char * toBuffer)
{
   // we have 2 states: (1) we are inside a row or (2) we are not.
   // to determine we are in a row we need to find "<td>", to determine
   // we are out we need to find </td>.
   bool inRow = FALSE;
   int bufflen = strlen(fromBuff);
   int i,j;
   i = j = 0;
   for ( i=0; i<bufflen; i++) {
      if ( (!inRow) && (fromBuff[i] == '<') && (fromBuff[i + 1] == 't') && 
           (fromBuff[i + 2] == 'r') && (fromBuff[i + 3] == '>') ) {
         // we have found a cell and should start recording 
         inRow = TRUE;
         toBuffer[j++] = fromBuff[i];
      }
      else if ( (inRow) && (fromBuff[i] == '>') && (fromBuff[i - 1] == 'r') &&
                (fromBuff[i - 2] == 't') && (fromBuff[i - 3] == '/') ) {   
         inRow = FALSE;
         toBuffer[j++] = fromBuff[i];
         toBuffer[j++] = '\n';
      }
      else if ( inRow ) {
         // we will turn any newlines we find into spaces
         if ( fromBuff[i] == '\n' ) toBuffer[j++] = ' ';
         else toBuffer[j++] = fromBuff[i];
      }
   }
   toBuffer[j] = '\0';
}

